@@ -1,10 +1,11 @@
 # -*- coding: utf-8 -*-
 import uuid as uuid
 from datetime import datetime
-
-from pydantic import BaseModel, UUID4, Field, Extra, PrivateAttr
 from typing import Mapping, Optional, Any, Union, Dict
 
+from frozendict import frozendict
+from pydantic import BaseModel, UUID4, Field, PrivateAttr, ConfigDict
+
 
 class TaskDecodeError(ValueError):
     """
@@ -31,22 +32,40 @@ class TaskDecodeError(ValueError):
 
 
 class TaskMetadata(BaseModel):
-    not_before: Optional[datetime]
-    expires: Optional[datetime]
-    max_retries: Optional[int]
+    not_before: Optional[datetime] = None
+    expires: Optional[datetime] = None
+    max_retries: Optional[int] = None
     attempt: int = 0
-    scheduled: Optional[datetime]
-    received: Optional[datetime]
-    queue_name: Optional[str]
-    extra: Optional[Mapping[str, Union[int, float, bool, str, None]]]
+    scheduled: Optional[datetime] = None
+    received: Optional[datetime] = None
+    queue_name: Optional[str] = None
+    extra: Optional[Mapping[str, Union[int, float, bool, str, None]]] = None
+
+    def freeze(self):
+        return FrozenTaskMetadata.model_construct(**self.model_dump())
+
+
+class FrozenTaskMetadata(TaskMetadata):
+    model_config = ConfigDict(frozen=True)
 
 
 class Task(BaseModel):
     uuid: UUID4 = Field(default_factory=uuid.uuid4)
     meta: TaskMetadata = TaskMetadata()
     task_type: str
-    correlation_id: Optional[str]
-    payload: Any
+    correlation_id: Optional[str] = None
+    payload: Optional[Any] = None
 
     # this field SHOULD NOT be transferred
     _local: Dict = PrivateAttr(default_factory=dict)
+
+    def freeze(self):
+        data = self.model_dump(exclude={'meta'})
+        data['meta'] = self.meta.freeze()
+        data['_local'] = frozendict(self._local)
+        return FrozenTask.model_construct(**data)
+
+
+class FrozenTask(Task):
+    model_config = ConfigDict(frozen=True)
+
