@@ -6,7 +6,7 @@ import weakref
 from typing import Literal, Callable, Dict, List
 
 import logging
-from pydantic import ValidationError, BaseModel
+from pydantic import ValidationError, BaseModel, ConfigDict
 
 from patchwork.core.utils import cached_property, Flag
 
@@ -29,6 +29,8 @@ class Component:
 
         :cvar enabled: determines if component is enabled or not
         """
+        model_config = ConfigDict(extra='forbid')
+
         enabled: bool = True
 
     EVENTS = ('startup', 'shutdown')
